########################################################################
#  BUILD.MAK - MAKEFILE to build each of the distribution diskettes.   #
#                                                                      #
#  Author:  g.p. "Jerry" Honeycutt Jr.                                 #
#    Date:  2 October 1990                                             #
#                                                                      #
#  DESCRIPTION                                                         #
#                                                                      #
#    BUILD.MAK builds each of the distribution diskettes given in the  #
#    diskette LOCKLIST.  The environment is validated in BUILD.MAK,    #
#    thus the MAKEFILEs used assume the environment is valid.          #
#                                                                      #
#  MODIFICATION HISTORY                                                #
#                                                                      #
#    gjph - 10/02/90:  Initial revision                                #
########################################################################

########################################################################
#  MAKEFILE directives                                                 #
########################################################################

.MEMSWAP                #  Memory swapping, we use alot of memory.
.IGNORE                 #  Build over errors.
.SILENT                 #  Don't echo commands.        
.SUFFIXES               #  Disable internal rules.

%if %exists( $(RULE_LIST) )     #  Include the build rules.
.INCLUDE $(RULE_LIST)   
%endif

########################################################################
#  Setup the MAKEFILE switches if we are debugging or building all.    #
########################################################################

%if "$(DEBUG)" != ""
MODES += -d DEBUG=TRUE
%endif
%if "$(ALL)" != ""
MODES += -a ALL=TRUE
%endif

########################################################################
#  Get the name of each distribution diskette for the LOCKLIST.        #
########################################################################

%if %exists( $(DISK_LIST) )
DISKETTES = $[@,$(DISK_LIST)]
%endif

########################################################################
#  The PROLOG of BUILD.MAK validates the environment.  In addition,    #
#  it deletes the log files if they exist and displays the name.       #
########################################################################

.PROLOG:
        cls
        %if "$(THIS_PRODUCT)"   == "" ||\
            "$(BASELINE)"       == "" ||\
            "$(LOCAL)"          == "" ||\
            "$(SYSTEM_AREA)"    == "" ||\
            "$(DISK_SET)"       == "" ||\
            "$(LOCK_SET)"       == "" ||\
            "$(RULE_LIST)"      == "" ||\
            "$(MODULE_LIST)"    == "" ||\
            "$(DISK_LIST)"      == "" ||\
            "$(OFFSET_SOURCE)"  == "" ||\
            "$(OFFSET_TARGET)"  == "" ||\
            "$(LOG_BUILD)"      == "" ||\
            "$(LOG_ERROR)"      == ""
                %echo Environment variables are not properly defined.
                %abort
        %endif
        %if "$(DEPEND_LIST)"    == ""
                %echo No dependency list is given; build will continue.
                pause >NUL
        %endif
        %if !%exists( $(DEPEND_LIST) )
                %echo $(DEPEND_LIST) does not exist; build will continue.
                pause >NUL
        %endif
        %if !%exists( $(RULE_LIST) )
                %echo $(RULE_LIST) does not exist.
                %abort
        %endif
        %if !%exists( $(MODULE_LIST) )
                %echo $(MODULE_LIST) does not exist.
                %abort
        %endif
        %if !%exists( $(DISK_LIST) )
                %echo $(DISK_LIST) does not exist.
                %abort
        %endif
        %if %exists( $(LOG_ERROR) )
                del $(LOG_ERROR) >NUL
        %endif
        %if %exists( $(LOG_BUILD) )
                del $(LOG_BUILD) >NUL
        %endif
        cls
        %cd $(BASELINE)
        echo $(THIS_PRODUCT) >>$(LOG_BUILD)
        echo ------------------------------------------------------------------------ >>$(LOG_BUILD)
        %echo $(THIS_PRODUCT)
        %echo ------------------------------------------------------------------------
        %echo 

########################################################################
#  The EPILOG returns us to the directory we started in.               #
########################################################################

.EPILOG:
        %cd $(CWD)

########################################################################
#  The targets we are building are each of the distribution diskettes. #
########################################################################

head:   $(DISKETTES)

########################################################################
#  For each of the distribution diskettes, we need to invoke another   #
#  make to build the files for the diskette.                           #
########################################################################

dummy:

$(DISKETTES):  dummy
        &=make DISK=$@ $(MODES) -f $(SYSTEM_AREA)\DISK.MAK

########################################################################
#  Include the extra dependencies.                                     #
########################################################################

%if %exists( $(DEPEND_LIST) )
.INCLUDE $(DEPEND_LIST)         #  Include the extra dependencies.
%endif

########################################################################
#  End BUILD.MAK                                                       #
########################################################################

