########################################################################
#  DISK.MAK - MAKEFILE to build each of the files for the diskette.    #
#                                                                      #
#  Author:  g.p. "Jerry" Honeycutt Jr.                                 #
#    Date:  2 October 1990                                             #
#                                                                      #
#  DESCRIPTION                                                         #
#                                                                      #
#    DISK.MAK builds each of the files belonging to the distribution   #
#    diskette.  The locklist for the diskette has the same name as     #
#    the diskette.  Thus, diskette names are limited to 8 characters   #
#    plus an extension.                                                #
#                                                                      #
#  MODIFICATION HISTORY                                                #
#                                                                      #
#    gjph - 10/02/90:  Initial revision                                #
########################################################################

########################################################################
#  MAKEFILE directives                                                 #
########################################################################

.MEMSWAP                #  Memory swapping, we use alot of memory.
.IGNORE                 #  Build over errors.
.SILENT                 #  Don't echo commands.        
.SUFFIXES               #  Disable internal rules.

%if %exists( $(RULE_LIST) )     #  Include the build rules.
.INCLUDE $(RULE_LIST)   
%endif

########################################################################
#  Setup the MAKEFILE switches if we are debugging or building all.    #
########################################################################

%if "$(DEBUG)" != ""
MODES += -d DEBUG=TRUE
%endif
%if "$(ALL)" != ""
MODES += -a ALL=TRUE
%endif

########################################################################
#  Get the name of each file for the diskette from the LOCKLIST.       #
#  The name includes the path.                                         #
########################################################################

%if %exists( $(LOCK_SET)\$(DISK) )
FILES = $[f,$(DISK_SET)\$(DISK),$[@,$(LOCK_SET)\$(DISK)]]
%endif

########################################################################
#  Describe what we are making.                                        #
########################################################################

.PROLOG:
        cls
        %cd $(BASELINE)
        echo -- $(DISK) >>$(LOG_BUILD)
        %echo $(DISK)
        %echo ------------------------------------------------------------------------
        %echo 

########################################################################
#  Return us to the directory we came from.                            #
########################################################################

.EPILOG:
        %cd $(CWD)

########################################################################
#  The targets we are building are each of the files for the disk.     #
########################################################################

head:   $(FILES)

########################################################################
#  For each of the files on the diskette, we need to invoke another    #
#  make to build the components each file is made from.                #
########################################################################

dummy:

$(FILES):  dummy
        &=make FILE=$@ $(MODES) -f $(SYSTEM_AREA)\FILE.MAK 

########################################################################
#  Include the extra dependencies.                                     #
########################################################################

%if %exists( $(DEPEND_LIST) )
.INCLUDE $(DEPEND_LIST)         #  Include the extra dependencies.
%endif

########################################################################
#  End DISK.MAK                                                        #
########################################################################

