########################################################################
#  FILE.MAK - MAKEFILE to build each of the source files required.     #
#                                                                      #
#  Author:  g.p. "Jerry" Honeycutt Jr.                                 #
#    Date:  2 October 1990                                             #
#                                                                      #
#  DESCRIPTION                                                         #
#                                                                      #
#    FILE.MAK builds each of the source component files that are       #
#    required to make the file.  The component file names are          #
#    retrieved from the file LOCKLIST by the same name.  This          #
#    MAKEFILE is recursive-each component file in turn can depend      #
#    on even more component files.                                     #
#                                                                      #
#    -------------------------------------------------------------     #
#    NOTE:  For this MAKEFILE to place the component files             #
#           in their correct location, they must be SEEDED.  See       #
#           the article text for an explanation.                       #
#    -------------------------------------------------------------     #
#                                                                      #
#  MODIFICATION HISTORY                                                #
#                                                                      #
#    gjph - 10/02/90:  Initial revision                                #
########################################################################

########################################################################
#  MAKEFILE directives                                                 #
########################################################################

.MEMSWAP                #  Memory swapping, we use alot of memory.
.IGNORE                 #  Build over errors.
.SILENT                 #  Don't echo commands.        
.SUFFIXES               #  Disable internal rules.

%if %exists( $(RULE_LIST) )     #  Include the build rules.
.INCLUDE $(RULE_LIST)   
%endif

########################################################################
#  Setup the MAKEFILE switches if we are debugging or building all.    #
########################################################################

%if "$(DEBUG)" != ""
MODES += -d DEBUG=TRUE
%endif
%if "$(ALL)" != ""
MODES += -a ALL=TRUE
%endif

########################################################################
#  Get the name of each source component file that is required to make #
#  the file.  Also, get the paths of each of the logical modules in    #
#  in the system.                                                      #
########################################################################

%if %exists( $(LOCK_SET)\$[r,$(FILE)].$[e,$(FILE)] )
COMPONENTS = $[@,$(LOCK_SET)\$[r,$(FILE)].$[e,$(FILE)]]
%endif
%if %exists( $(MODULE_LIST) )
MODULES = $[@,$(MODULE_LIST)]
%endif

########################################################################
#  Specify were all of the files required can be found.                #
########################################################################

.SOURCE : \
%foreach MODULE in $(MODULES)
                $(BASELINE)\$(MODULE) \
                $(BASELINE)\$(MODULE)\$(OFFSET_SOURCE) \
                $(BASELINE)\$(MODULE)\$(OFFSET_TARGET) \
%endfor

########################################################################
#  Describe what we are making.                                        #
########################################################################

.PROLOG:
        cls
        %cd $(BASELINE)
        %echo $(FILE)
        %echo ------------------------------------------------------------------------
        %echo 

########################################################################
#  Return us to the directory we came from.                            #
########################################################################

.EPILOG:
        %cd $(CWD)

########################################################################
#  The targets we are building are each of the source components.      #
########################################################################

head:           $(FILE)

$(FILE):        $(COMPONENTS)

########################################################################
#  For each of the source components, we need to invoke yet another    #
#  make to build them.                                                 #
########################################################################

dummy:

$(COMPONENTS):  dummy
        &=make FILE=$@ $(MODES) -f $(SYSTEM_AREA)\FILE.MAK 

########################################################################
#  Include the extra dependencies.                                     #
########################################################################

%if %exists( $(DEPEND_LIST) )
.INCLUDE $(DEPEND_LIST)         #  Include the extra dependencies.
%endif

########################################################################
#  End FILE.MAK                                                        #
########################################################################

