########################################################################
#  LOCAL.MAK - MAKEFILE to build a given file locally.                 #
#                                                                      #
#  Author:  g.p. "Jerry" Honeycutt Jr.                                 #
#    Date:  2 October 1990                                             #
#                                                                      #
#  DESCRIPTION                                                         #
#                                                                      #
#    LOCAL.MAK builds a specified file on the LOCAL work area.  The    #
#    file is built from any files required that are found in the       #
#    LOCAL work area and then any other references are resolved        #
#    against the BASELINE work area.  In other words, the code that    #
#    is being developed locally is linked to the rest of the code      #
#    that already exists on the BASELINE work area.                    #
#                                                                      #
#    Usage:  MAKE FILE=<TARGET> -F LOCAL.MAK                           #
#                                                                      #
#                                                                      #
#  MODIFICATION HISTORY                                                #
#                                                                      #
#    gjph - 10/02/90:  Initial revision                                #
########################################################################

########################################################################
#  MAKEFILE directives                                                 #
########################################################################

.MEMSWAP                #  Memory swapping, we use alot of memory.
.IGNORE                 #  Build over errors.
.SILENT                 #  Don't echo commands.        
.SUFFIXES               #  Disable internal rules.

%if %exists( $(RULE_LIST) )     #  Include the build reuls.
.INCLUDE $(RULE_LIST)   
%endif

########################################################################
#  Setup the MAKEFILE switches if we are debugging or building all.    #
########################################################################

%if "$(DEBUG)" != ""
MODES += -d DEBUG=TRUE
%endif
%if "$(ALL)" != ""
MODES += -a ALL=TRUE
%endif

########################################################################
#  Get the list of all the source components in the system.  This is   #
#  used to find the files to link against.  Also, get each source      #
#  component that is required to build the given file.                 #
########################################################################

%if %exists( $(LOCK_SET)\$[r,$(FILE)].$[e,$(FILE)] )
COMPONENTS = $[@,$(LOCK_SET)\$[r,$(FILE)].$[e,$(FILE)]]
%endif
%if %exists( $(MODULE_LIST) )
MODULES = $[@,$(MODULE_LIST)]
%endif

########################################################################
#  Specify were all of the files required can be found.                #
########################################################################

.SOURCE : $(LOCAL)\
%foreach MODULE in $(MODULES)
                $(BASELINE)\$(MODULE) \
                $(BASELINE)\$(MODULE)\$(OFFSET_TARGET) \
%endfor

########################################################################
#  Describe what we are making.                                        #
########################################################################

.PROLOG:
        cls
        %if "$(THIS_PRODUCT)"   == "" ||\
            "$(BASELINE)"       == "" ||\
            "$(LOCAL)"          == "" ||\
            "$(SYSTEM_AREA)"    == "" ||\
            "$(DISK_SET)"       == "" ||\
            "$(LOCK_SET)"       == "" ||\
            "$(RULE_LIST)"      == "" ||\
            "$(MODULE_LIST)"    == "" ||\
            "$(DISK_LIST)"      == "" ||\
            "$(OFFSET_SOURCE)"  == "" ||\
            "$(OFFSET_TARGET)"  == "" ||\
            "$(LOG_BUILD)"      == "" ||\
            "$(LOG_ERROR)"      == ""
                %echo Environment variables are not properly defined.
                %abort
        %endif
        %if !%exists( $(RULE_LIST) )
                %echo $(RULE_LIST) does not exist.
                %abort
        %endif
        %if "$(DEPEND_LIST)"    == "" && "$(RECURSE)" == ""
                %echo No dependency list is given; build will continue.
                pause >NUL
        %endif
        %if !%exists( $(DEPEND_LIST) ) && "$(RECURSE)" == ""
                %echo $(DEPEND_LIST) does not exist; build will continue.
                pause >NUL
        %endif
        %if !%exists( $(MODULE_LIST) )
                %echo $(MODULE_LIST) does not exist.
                %abort
        %endif
        %if !%exists( $(DISK_LIST) )
                %echo $(DISK_LIST) does not exist.
                %abort
        %endif
        %if %exists( $(LOG_ERROR) ) && "$(RECURSE)" == ""
                del $(LOG_ERROR) >NUL
        %endif
        %if %exists( $(LOG_BUILD) ) && "$(RECURSE)" == ""
                del $(LOG_BUILD) >NUL
        %endif
        cls
        %cd $(LOCAL)
        %echo $(FILE)
        %echo ------------------------------------------------------------------------
        %echo 

########################################################################
#  Return us to the directory we came from.                            #
########################################################################

.EPILOG:
        %cd $(CWD)

########################################################################
#  The target we are building is the single file specified.            #
########################################################################

head:           $(FILE)

$(FILE):        $(COMPONENTS)

########################################################################
#  For each of the source componenets, we need to invoke yet another   #
#  make to build them.                                                 #
########################################################################

dummy:

$(COMPONENTS):  dummy
        &=make RECURSE=TRUE FILE=$(LOCAL)\$[r,$@].$[e,$@] $(MODES) -f LOCAL.MAK 

########################################################################
#  Include the extra dependencies.                                     #
########################################################################

%if %exists( $(DEPEND_LIST) )
.INCLUDE $(DEPEND_LIST)         #  Include the extra dependencies.
%endif

########################################################################
#  End LOCAL.MAK                                                       #
########################################################################

