IMPLEMENTATION MODULE Support;

 (*  File: Support.mod  Author:  Michael F. Bisignani  June 89  *)
 
FROM Csqueue IMPORT 
    InsertCSQueue,RemoveCSQueue;
FROM InOut IMPORT WriteString,WriteLn,WriteInt,
                  OpenOutput,CloseOutput;
FROM RealInOut IMPORT WriteReal;
FROM Elements IMPORT ElementType;
FROM Procs IMPORT process1,process2,process3;
FROM Cglobals IMPORT 
     (*CONSTANTS*)
     NUMPROCS,MAXRUNTIME,
     (* TYPES  *)
     StateType,Semaphore,ProcIdRange,String10, 
     (* VARIABLES  *)
     ProcessTable,Mutex,SharedVariable,SysErrorNo,SchedProc,MainProc,
     ReadyQueue,BlockedQueue,Scheduling,ct;
FROM SYSTEM IMPORT ADR,SIZE,NEWPROCESS,
          TRANSFER,ENABLE,DISABLE;
      
(*---------------------------------------------*)
PROCEDURE UseSharedData;
BEGIN
  idle(1000);
  SharedVariable := SharedVariable+0.1;
  idle(10000);
END UseSharedData;
(*----------------------------------------------*)
PROCEDURE CallExit(Pid:INTEGER);
 BEGIN
  ProcessTable[Pid].State := DONE;
  WHILE ProcessTable[Pid].State = DONE DO END; 
 END CallExit;
(*----------------------------------------------*)
PROCEDURE idle(tlen:INTEGER);
 VAR i:INTEGER;
BEGIN
  FOR i := 1 TO tlen DO END;   
END idle;
(*------------------------------------------------*)
PROCEDURE EnterCriticalSection(Pid:INTEGER);
BEGIN
   P(Mutex,Pid); 
END  EnterCriticalSection;
(*------------------------------------------------*)
PROCEDURE LeaveCriticalSection;
BEGIN
   V(Mutex);
END LeaveCriticalSection;
(*-----------------------------------------------*)
PROCEDURE P(VAR s:Semaphore;Pid:INTEGER);
BEGIN
  DISABLE;
  DEC(s);
  IF s < 0 
   THEN 
     PutToSleep(Pid); (*  The Calling Process Will Be Blocked Here *)
     (* Interrupts Will have Been Enabled When it Returns *)
   ELSE 
     ENABLE;  (*  Enable Interrupts Again *)  
   END;  
END P;
(*---------------------------------------------*)
PROCEDURE  V(VAR s:Semaphore);
BEGIN
  DISABLE;
  INC(s);
  IF s <=0 THEN (*  The Must Still Be Some Process Sleeping on S *)
    WakeUpSomeProcess;
  END;  
  ENABLE;
END V;
(*----------------------------------------------*)
PROCEDURE PutToSleep(Pid:INTEGER);
BEGIN
  (* Tell The Scheduler That This One Must Sleep *)
  ProcessTable[Pid].MustBeBlocked := TRUE;
  (*  Busy Wait Until The Next Interrupt at which 
      time the Scheduler Will Put This Process on 
      the Blocked Queue , but Remember to ENABLE *)     
   ENABLE;    
   WHILE ProcessTable[Pid].MustBeBlocked DO END ; 
  END PutToSleep;
(*---------------------------------------------*)
PROCEDURE WakeUpSomeProcess;
VAR TempId  : ElementType;
BEGIN
 IF RemoveCSQueue(BlockedQueue,TempId)
   THEN 
     (*  Un Do the Flag, The Sleeping Process Can Now Proceed *)
     WITH ProcessTable[TempId.ProcessId] DO 
       MustBeBlocked := FALSE;
       State := READY;
     END;  
     IF NOT InsertCSQueue(ReadyQueue,TempId)
         THEN HALT;  END;
   ELSE Panic(4);  END;
  END WakeUpSomeProcess;
(*----------------------------------------------*)
PROCEDURE Panic(Code:INTEGER);
BEGIN
  SysErrorNo := Code;
  TRANSFER(SchedProc,MainProc);
END Panic;
(*------------------------------------------------*)
PROCEDURE ErrorMsg(Code:INTEGER);
BEGIN
   CASE Code OF 
     0 :  WriteString("0 - Normal Termination")|
     1 :  WriteString("1 - Empty Ready Queue and NOT ALLBLOCKED")|
     2 :  WriteString("2 - Blocked Queue FULL")|
     3 :  WriteString("3 - Ready Queue FULL")|
     4 :  WriteString("4 - Blocked Queue Empty - Expected Process")|
     5 :  WriteString("5 - DeadLock Detected");
     ELSE WriteString("Unkown ErrorNo - No Descpriptor Available");
   END; (*  CASE  *);
   WriteLn;   
END ErrorMsg;
(*--------------------------------------------------*)
PROCEDURE AllBlocked():BOOLEAN;
 VAR i: ProcIdRange;
BEGIN
  FOR i := 1 TO NUMPROCS DO 
    IF ProcessTable[i].State <> BLOCKED THEN
      RETURN(FALSE);
    END;
  END;
  RETURN(TRUE);    
END AllBlocked;
(*----------------------------------------------------------------*)
PROCEDURE RecordScheduling(ScheduledProc:INTEGER;SV:INTEGER);
BEGIN
   INC(Scheduling.Count);
   WITH Scheduling.Data[Scheduling.Count] DO 
       ProcessId := ScheduledProc;
       SValue    := SV;
   END;    
END   RecordScheduling;
(*------------------------------------------------------------*)
PROCEDURE DisplayEvents;
 VAR i:  [1..MAXRUNTIME*2];
BEGIN
 (* Note: This Procedure Is Called After The MultiProgramming *)
 (* has terminated, therefore it is safe to use the Standard  *)
 (* I/O Routines*)
 (* Redirect Output to a File with extension .run *)
 OpenOutput("run");    
 (* How did The Program Terminate*)
 ErrorMsg(SysErrorNo);
 (* The  Shared Variable  *)
 WriteString("The Shared Variable ");
 WriteReal(SharedVariable,12);
 WriteLn;WriteLn;
 (*  Show The Counters Which Each Process Has Modified *)
 WriteString("The Local Counters ");WriteLn;
 FOR i := 1 TO NUMPROCS DO 
    WriteString("Ct[");WriteInt(i,3);
    WriteString("] "); WriteInt(ct[i],10);WriteLn;
 END;
 WriteLn;WriteLn;
 WriteString("The Scheduling and Semaphore Values ");WriteLn; 
 IF Scheduling.Count > 0 
  THEN
    WITH Scheduling DO 
      FOR i := 1 TO  Count DO
          WriteString("Time ");WriteInt(i,3);WriteString(": ");
          WriteInt(Data[i].ProcessId,3);WriteInt(Data[i].SValue,3);
          WriteLn;
      END; (* FOR *)
    END;  (*  WITH  *)
  ELSE
    WriteString("No Events Recorded");         
  END;
  (*  Stop Output Redirection *)  
  CloseOutput;
END DisplayEvents;
(*-----------------------------------------------*)
PROCEDURE StartUp;
  VAR i: ProcIdRange;
      ProcessPtr: PROC ;   (*  This is a pointer *)
  
  PROCEDURE AssignProcedurePtr(ProcNo:INTEGER;VAR Ptr:PROC);
   BEGIN
     CASE ProcNo OF 
       1 : Ptr := process1  |
       2 : Ptr := process2  |
       3 : Ptr := process3  ;
     END;  (*   CASE  *)
   END AssignProcedurePtr;  (*  Nested  *)
BEGIN
  (*  First All Process Coroutines *)
  FOR i := 1 TO NUMPROCS DO
    (*  Set Up the Procedure Pointer *)
    AssignProcedurePtr(i,ProcessPtr);  
    (*  Allocate The Coroutine WorksSpace*)
    NEWPROCESS(ProcessPtr,ADR(ProcessTable[i].WorkSpace),
              SIZE(ProcessTable[i].WorkSpace),ProcessTable[i].Context);
    (* Initialize The Process Table  *) 
    WITH ProcessTable[i] DO 
      State := READY;
      MustBeBlocked := FALSE;
      PID.ProcessId := INTEGER(i);  (*  For Simplicity *)
      IF InsertCSQueue(ReadyQueue,PID) THEN END;
    END;  (* WITH  *)  
    ct[i] := 0; (*  Initialize the Local Counter *)
  END;  
END StartUp;
BEGIN  (*  Initialization *)

END Support.  
